#include "config.h"
#include <stdio.h>
#include "3d.h"
#include "layout.h"

#define SQRT2 1.41421356
#define SQRT3 1.7320508
#define SIN_45 (1.0/SQRT2)
#define COS_45 (1.0/SQRT2)
#define SIN_60 (SQRT3/2.0)
#define COS_60 (1/2.0)
#define SIN_30 (1/2.0)
#define COS_30 (SQRT3/2.0)

point2d_t tr3d2d_top(point3d_t p3d){
    point2d_t p2d;
    p2d.x =  (p3d.x*TOPVIEW_WIDTH/2)/XMAX + TOPVIEW_X0 + TOPVIEW_XMARGIN/2;
    p2d.y = -(p3d.z-ZMIN)*(WIN_HEIGHT-TOPVIEW_YMARGIN*2)/ZSIZE
            +WIN_HEIGHT - TOPVIEW_YMARGIN;
    return p2d;
}

point2d_t tr3d2d_isom(point3d_t p3d){
    point2d_t p2d, p2diso;
    int h;
    
    // 原点中心のトップビューを作成
    p2d.x =  (p3d.x*ISOMVIEW_WIDTH/2)/(XMAX*1.3);
    p2d.y = -(p3d.z-ZMIN)*(WIN_HEIGHT-ISOMVIEW_YMARGIN*2)/(ZSIZE-128)
            + WIN_HEIGHT/2;
    
#if 1
    p2diso.x = p2d.x*COS_45 - p2d.y*SIN_45;
    p2diso.y = p2d.x*SIN_45 + p2d.y*COS_45;
    p2diso.y *= COS_60;
    h = ISOM_HEIGHT*(p3d.y-GROUND_HEIGHT)/FLIGHT_HEIGHT;
    printf("h:%d\n", h);
    p2diso.y += h;
#else    
    p2diso.x = p2d.x;
    p2diso.y = p2d.y;
#endif
    // 画面内に収まるように水平移動する
    p2diso.x +=  ISOMVIEW_X0 + ISOMVIEW_XMARGIN;
    p2diso.y += WIN_HEIGHT/2 + ISOMVIEW_YMARGIN;

    

    // + TOPVIEW_X0 + TOPVIEW_XMARGIN/2;  
        //            +WIN_HEIGHT - TOPVIEW_YMARGIN;
  //  p2d.x = p3d.x * R / p3d.z + X0;
  //  p2d.y = p3d.y * R / p3d.z + Y0;
  return p2diso;
}

point2d_t tr3d2d(point3d_t p3d){
    point2d_t p2d;
    p2d.x = p3d.x * R / p3d.z + X0;
    p2d.y = p3d.y * R / p3d.z + Y0;
    return p2d;
}

point2d_t tr3d2d_org(point3d_t p3d, int x0, int y0){
    point2d_t p2d;
    p2d.x = p3d.x * R / p3d.z + x0;
    p2d.y = p3d.y * R / p3d.z + y0;
    return p2d;
}

point3d_t add_vector3d(point3d_t pos1, point3d_t pos2){
  point3d_t ret;
  ret.x = pos1.x + pos2.x;
  ret.y = pos1.y + pos2.y;
  ret.z = pos1.z + pos2.z;
  return ret;
}


point3d_t reverse_vector3d(point3d_t pos){
  point3d_t ret;
  ret.x = -pos.x;
  ret.y = -pos.y;
  ret.z = -pos.z;
  return ret;
}

point3d_t rot_zy(point3d_t p3d, int d){
  double r = Deg2Rad(d);
  point3d_t new_p3d;

  new_p3d.z = (int)(p3d.z * cos(r) - p3d.y * sin(r));
  new_p3d.y = (int)(p3d.z * sin(r) + p3d.y * cos(r));
  new_p3d.x = p3d.x;
  return new_p3d;
}

point3d_t rot_xy(point3d_t p3d, int d){
  double r = Deg2Rad(d);
  point3d_t new_p3d;

  new_p3d.x = (int)(p3d.x * cos(r) - p3d.y * sin(r));
  new_p3d.y = (int)(p3d.x * sin(r) + p3d.y * cos(r));
  new_p3d.z = p3d.z;
  return new_p3d;
}

point3d_t rot_xz(point3d_t p3d, int d){
  double r = Deg2Rad(d);
  point3d_t new_p3d;

  new_p3d.x = (int)(p3d.x * cos(r) - p3d.z * sin(r));
  new_p3d.z = (int)(p3d.x * sin(r) + p3d.z * cos(r));
  new_p3d.y = p3d.y;
  return new_p3d;
}

point3d_t resize(point3d_t p3d, double scale){
  p3d.x = (int)(p3d.x * scale);
  p3d.y = (int)(p3d.y * scale);
  p3d.z = (int)(p3d.z * scale);
  return p3d;
}

void resize_line(point3d_t *st, point3d_t *en, double scale){
    point3d_t mid;
    mid.x = (st->x + en->x) / 2;
    mid.y = (st->x + en->y) / 2;
    mid.z = (st->x + en->z) / 2;
    st->x = (int)(mid.x + (st->x - mid.x) * scale);
    st->y = (int)(mid.y + (st->y - mid.y) * scale);
    st->z = (int)(mid.z + (st->z - mid.z) * scale);
    en->x = (int)(mid.x + (en->x - mid.x) * scale);
    en->y = (int)(mid.y + (en->y - mid.y) * scale);
    en->z = (int)(mid.z + (en->z - mid.z) * scale);
}
