#include "config.h"
#include <stdio.h>
#include <EzGraph.h>
#include "3ddraw.h"
#include "3dshoot.h"
#include "color.h"
#include "rnd.h"

/*
奥行きがｚ軸、高さ方向がｙ軸

*/

int sensor_motion = 0;
static view_mode_t view_mode = VIEW_MODE_DEFAULT;
static int sensor_x0, sensor_y0;


void set_view_mode(view_mode_t new_view_mode){
    printf("switch to view mode %d->%d\n", view_mode, new_view_mode);;
    view_mode = new_view_mode;
    switch (view_mode){
    case vm_bird:
        sensor_x0 = SENSOR_X0;
        sensor_y0 = SENSOR_Y0;
        break;
    case vm_top:
        sensor_x0 = SENSOR_X0_TOP;
        sensor_y0 = SENSOR_Y0_TOP;
        break;
    case vm_isom:
        sensor_x0 = SENSOR_X0_ISO;
        sensor_y0 = SENSOR_Y0_ISO;
        break;
    default:
        assert(0);
    }
}

view_mode_t get_view_mode(void){
    return view_mode;
}


void draw_3dline(point3d_t st3d, point3d_t en3d){
    point2d_t st2d, en2d;
    // 始点か終点のどちらかが視点より後方なら、この線分は描画しない
    // 本当はz=0平面でクリッピングすべきだが、z<=0となるのは爆発エフェクト
    // ぐらいのはずなので、線分全体を省略する。
    if (st3d.z <= 0 || en3d.z <= 0){
        return;
    }
    switch (view_mode){
    case vm_bird:
        st2d = tr3d2d(st3d);
        en2d = tr3d2d(en3d);
        break;
    case vm_top:
        st2d = tr3d2d_top(st3d);
        en2d = tr3d2d_top(en3d);
        break;
    case vm_isom:
        st2d = tr3d2d_isom(st3d);
        en2d = tr3d2d_isom(en3d);
        break;
    default:
        assert(0);
        break;
    }
    EzDrawLineB(st2d.x, st2d.y, en2d.x, en2d.y);
}

void draw_3dline_sensor(point3d_t st3d, point3d_t en3d,	int xa, int ya){
    point2d_t st2d, en2d;
    /*
    point2d_t lt = {SENSOR_LT_X, SENSOR_LT_Y};
    point2d_t rb = {SENSOR_RB_X, SENSOR_RB_Y};
    */

    st2d = tr3d2d_org(st3d, sensor_x0, sensor_y0);
    en2d = tr3d2d_org(en3d, sensor_x0, sensor_y0);

    /* 今のところセンサー内の描画は枠外にかからないので */
    /* クリッピングは省略する */
    EzDrawLineB(st2d.x + xa, st2d.y + ya, en2d.x + xa, en2d.y + ya);
    /*
    if (clip_line(&st2d, &en2d, lt, rb)){
        EzDrawLineB(st2d.x + xa, st2d.y + ya, en2d.x + xa, en2d.y + ya);
    }
    */
}

void draw_shadowline(point3d_t st3d, point3d_t en3d){
  st3d.y = GROUND_HEIGHT;
  en3d.y = GROUND_HEIGHT;
  draw_3dline(st3d, en3d);
}

void draw_model(model3d_t *model, int motion, point3d_t pos){
    int i;

    //    SetColor(model->color);
    if (set_color(model->color_type)) return;
    for (i = 0 ; i < model->edge_num; i++){
        draw_3dline(add_vector3d(model->vertex[motion][model->edge[i].st],
                                 pos),
                    add_vector3d(model->vertex[motion][model->edge[i].en],
                                 pos));
    }
}

void draw_hitbox(model3d_t *model, point3d_t pos){
    point3d_t pss, psl, pls, pll;

    pss = psl = pls = pll = pos;
    pss.y = psl.y = pls.y = pll.y = FLIGHT_HEIGHT;
    pss.x -= model->xsize;
    psl.x -= model->xsize;
    pss.z -= model->zsize;
    psl.z += model->zsize;
    pls.x += model->xsize;
    pll.x += model->xsize;
    pls.z -= model->zsize;
    pll.z += model->zsize;
    //    SetColor(3);
    set_color(ct_hitbox);
    draw_3dline(pss, psl);
    draw_3dline(psl, pll);
    draw_3dline(pll, pls);
    draw_3dline(pls, pss);
}

void draw_shadow(model3d_t *model, int motion, point3d_t pos){
    int i;

    //    SetShadowColor();
    // 描画色がNO_DRAWなら描画処理をスキップ
    if (set_color(ct_shadow)) return;
    for (i = 0 ; i < model->sedge_num; i++){
        draw_shadowline(add_vector3d(model->vertex[motion][model->sedge[i].st],
                                     pos),
                        add_vector3d(model->vertex[motion][model->sedge[i].en],
                                     pos));
    }
}

void animate_sensor_model(model3d_t *model){
    static model3d_t *prev_model = NULL;

   // 前回と違うモデルのときはモーション番号を初期化
    if (prev_model != model){
        sensor_motion = model->motion_st;
        prev_model = model;
    }
    else {
        sensor_motion ++;
        if (sensor_motion > model->motion_en){
            sensor_motion = model->motion_st;
        }
    }
}

void draw_sensor_model(model3d_t *model){
    int i;
    //    static point3d_t pos = {0, GROUND_HEIGHT, 500};
    static point3d_t pos = {-400, 400, 300};
     //    static int motion = 0;

    //    SetColor(model->color);
    set_color(model->color_type);
    for (i = 0 ; i < model->edge_num; i++){
        draw_3dline_sensor(add_vector3d(model->vertex[sensor_motion][model->edge[i].st],
                                        pos),
                           add_vector3d(model->vertex[sensor_motion][model->edge[i].en],
                                        pos),
                           model->sensor_xoffset, model->sensor_yoffset);
    }
}


void draw_explosion(model3d_t *model, int motion, int emotion, point3d_t pos){
  int i;
  //  SetColor(model->color);
  set_color(model->color_type);
  //   printf("eedge_num:%d evertex_num:%d\n", model->eedge_num, model->evertex_num);
  for (i = 0 ; i < model->eedge_num; i++){
	draw_3dline(add_vector3d(model->evertex[motion][emotion]
                                           [model->eedge[i].st], pos),
				add_vector3d(model->evertex[motion][emotion]
                                           [model->eedge[i].en], pos));
  }
}

void create_explosion(model3d_t *model){
    int i, j, motion, emotion, noize;
    int st, en;

    noize = 2;
    for (motion = 0 ; motion < model->motion_num; motion ++){
        j = 0;
        for (i = 0 ; i < model->edge_num ; i++){
            st = model->edge[i].st;
            en = model->edge[i].en;
            model->evertex[motion][0][j].x = model->vertex[motion][st].x
                + RRnd(-noize, noize);
            model->evertex[motion][0][j].y = model->vertex[motion][st].y
                + RRnd(-noize, noize);
            model->evertex[motion][0][j].z = model->vertex[motion][st].z
                + RRnd(-noize, noize);
            model->evertex[motion][0][j+1].x = model->vertex[motion][en].x
                + RRnd(-noize, noize);
            model->evertex[motion][0][j+1].y = model->vertex[motion][en].y
                + RRnd(-noize, noize);
            model->evertex[motion][0][j+1].z = model->vertex[motion][en].z
                + RRnd(-noize, noize);
            model->eedge[i].st = j;
            model->eedge[i].en = j+1;

              resize_line(&model->evertex[motion][0][j],
                        &model->evertex[motion][0][j+1], 0.5);
            //        printf("exp:(%d, %d %d)\n", model->evertex[0][j].x, model->evertex[0][j].y, model->evertex[0][j].z);
            j += 2;
        }
    }
    model->eedge_num = model->edge_num;
    model->evertex_num = model->edge_num*2;
    for (motion = 0 ; motion < model->motion_num; motion ++){
        for (emotion = 1 ; emotion < EMOTION_MAX; emotion ++){
            for (i = 0 ; i < model->evertex_num ; i+= 2){
                model->evertex[motion][emotion][i]
                    = model->evertex[motion][0][i];
                model->evertex[motion][emotion][i+1]
                    = model->evertex[motion][0][i+1];
                resize_line(&model->evertex[motion][emotion][i], 
                            &model->evertex[motion][emotion][i+1],
                            1.0/(emotion+1));
                model->evertex[motion][emotion][i]
                    = resize(model->evertex[motion][emotion][i], 
                             emotion + 1);
                model->evertex[motion][emotion][i+1] 
                    = resize(model->evertex[motion][emotion][i+1], 
                             emotion + 1);
            }
        }
    }
  /*
  noize = 5;
  for (i = 0 ; i < model->vertex_num ; i++){
	model->evertex[0][i].x = model->vertex[0][i].x + RRnd(-noize, noize);
	model->evertex[0][i].y = model->vertex[0][i].y + RRnd(-noize, noize);
	model->evertex[0][i].z = model->vertex[0][i].z + RRnd(-noize, noize);
  }
  for (motion = 1 ; motion < EMOTION_MAX; motion ++){
	for (i = 0 ; i < model->vertex_num ; i++){
	  model->evertex[motion][i] = model->evertex[0][i];
	  model->evertex[motion][i] = resize(model->evertex[motion][i], 
										 motion/1.5 + 1);
	}
  }
*/
}


