#ifndef D3MODEL_H
#define D3MODEL_H
#include "config.h"
#include "3d.h"
#include "color.h"

#define MOTION_MAX 32
#define EMOTION_MAX 16
#define VERTEX_MAX 32
#define EVERTEX_MAX (VERTEX_MAX*2)
#define EDGE_MAX   32
#define SEDGE_MAX  16
#define EEDGE_MAX  (EDGE_MAX*2)

typedef struct {
  int st;
  int en;
} lineix_t; 

typedef struct {
    color_type_t color_type;
    //    int color;
    point3d_t vertex[MOTION_MAX][VERTEX_MAX];
    point3d_t evertex[MOTION_MAX][EMOTION_MAX][EVERTEX_MAX];
    lineix_t edge[EDGE_MAX];
    lineix_t sedge[SEDGE_MAX];
    lineix_t eedge[EEDGE_MAX];
    int vertex_num;
    int evertex_num;
    int edge_num;
    int sedge_num;
    int eedge_num;
    // センサーのモーションループ開始・終了位置、モーションの総数
    // 状況に応じてモーションを使い分けるため、
    // 敵機のモーションはmove_xxx()内で制御するが、
    // センサーはこの範囲内でループさせる
    // 爆発エフェクト作成は全モーションについて作るため、総数が必要になる
    int motion_st, motion_en, motion_num;
    int xsize, zsize;    //当たり判定用サイズ
    int sensor_xoffset, sensor_yoffset;  //センサ表示時のオフセット
} model3d_t;

typedef enum {
    et_invalid = -1,
    et_start = 0,
    et_scout = 0,// 弾を撃たず左右に移動しつつ前進
    et_wall,     // 回転しつつ直進、弾をはじく
    et_drone,    // 自機の正面から避けるように移動
    et_fighter,  // 直進して正面弾を発射し、反転して退却する
    et_copter,   // 曲線コースを移動しつつ、ときどきねらい撃ち弾を発射
    et_lancer,   // X軸が合うように左右に加速しつつ体当たり
    et_walker,   // 2足歩行
    et_turret,   // 砲塔。正面弾を発射
    et_fortress, // ボス:画面上部を前後左右にゆっくり動きつつ弾を多数発射
    et_bomber,   // プロペラつき大型機
    et_byte,     // 小型浮遊砲
    et_gunroid,  // 戦闘ロボ
    et_wyvern,   // 羽ばたく有翼竜
    et_dragon,   // 多頭竜
    et_dragon_head, // 多頭竜の頭部
    et_core,     // 多頭竜二段階目
    et_end,
    et_midboss,  // 中ボス(ステージ構成登録用)
    et_boss,     // ボス(ステージ構成登録用)
    et_reserve = 63  // 将来の敵種追加時にachievementsの互換性がとれるように
                     // 64個分の領域を確保しておく
} enemytype_t;

extern model3d_t myship_model;
extern model3d_t enemyship_model[et_end];
extern model3d_t scout_model;
extern model3d_t wall_model;
extern model3d_t drone_model;
extern model3d_t fighter_model;
extern model3d_t copter_model;
extern model3d_t lancer_model;
extern model3d_t walker_model;
extern model3d_t turret_model;
extern model3d_t fortress_model;
extern model3d_t bomber_model;
extern model3d_t wyvern_model;
extern model3d_t byte_model;
extern model3d_t gunroid_model;
extern model3d_t dragon_head_model;
extern model3d_t dragon_model;
extern model3d_t core_model;

// etがボス・中ボスなら真
// BYTEはlife 1の雑魚扱い
#define IsEnemyTypeBoss(et) ((et) >= et_fortress && (et) != et_byte)

#define IsEnemyBlockingBullet(es) ((es)->type == et_wall || \
                                   ((es)->type == et_byte && (es)->motion != 8) )


#endif
