#ifndef RND_H
#define RND_H

/*
Xorshiftによる乱数関数
- srndは従来のsrandと同様に整数の種を与える。系列先頭の偏りを除去するため、
  内部的に適当な回数、乱数を生成して捨てる
- srnd4はXorshiftの乱数の種を直接設定する。
  4つとも0は不可であり、一部にあえて0を指定する必要もなさそうなので、
  0を指定した種は現在値のままとする
  ex. srnd4(123, 456, 0, 0); ... x=123, y=456とし、z, wはそのまま
*/
void srnd(unsigned int s);
void srnd4(unsigned long sx, unsigned long sy, unsigned long sz,
           unsigned long sw);
unsigned long rnd(void);

// min以上max以下の整数乱数を返す
// rrnd()関数のマクロ版
//#define Rnd(min, max) ((int)((double)rand()*((max)-(min)+1)/(RAND_MAX+1.0)) + (min))
#define RRnd(min, max) (rnd() % ((max)-(min)+1) + (min))
// 確率thrパーセントで非0を返す
// ※rnd()がunsignedなので、signedにキャストしておかないとProb(0)が1になる
#define Prob(thr) ((signed int)RRnd(0, 99) <= (thr)-1 ? 1 : 0)

// min以上max以下の整数乱数を返す
int rrnd(int min, int max);

#endif
